﻿(function () {

    "use strict";

    var resultScenarios = {
        none: 0,
        distinctLeader: 1,
        tie: 2,
        noResults: 3,
        notParticipating: 4,
        multipleLeaders: 5
    };

    angular.module("enrMapModule")
           .factory("mapRegionLeaderCalculator", ["officeJsonRegionFilter", "detailsData", "participationCalculator", function (officeJsonRegionFilter, detailsData, participationCalculator) {
               return {
                   resultScenarios: resultScenarios,

                   determineLeader: function (officeCategoryElectionResultsJson, regionProperties) {
                       var resultScenario;
                       var leaderPartyId = 0;

                       var filteredRegionNodes = [];
                       if (participationCalculator.isOfficesRegionParticipating(detailsData.getData(), regionProperties))
                           filteredRegionNodes = officeJsonRegionFilter.getRegionNodes(detailsData.getData(), regionProperties);

                       // If no region information is present, that region is not participating
                       if (filteredRegionNodes.length === 0)
                           resultScenario = resultScenarios.notParticipating;

                       // The presence of multiple nodes for a single region is not currently handled.
                       else if (filteredRegionNodes.length > 1)
                           throw Error("RegionNodes.length > 1 error in leader calculator for region with geoId: " + regionProperties.GEOID);

                       // We have found the 1 region node in JSON for this region
                       else {
                           var racesInThisRegion = filteredRegionNodes[0].Races.Race;

                           // Determine if there are multiple races for this office category in the region.
                           var isMultipleRaces = racesInThisRegion.length !== undefined;
                           if (isMultipleRaces) {

                               // It's possible the same race may be listed multiple times due the region having multiple reporting jurisdictions. Check for this.
                               var raceIdList = racesInThisRegion.map(function (race) { return race.OFFICEID });
                               var raceIdListDistinct = raceIdList.filter(function (value, index, self) { return self.indexOf(value) === index; });

                               if (raceIdListDistinct.length === 1) {

                                   // There is actually only one race. It's just listed multiple times.
                                   isMultipleRaces = false;

                                   // Rather than add up the votes (as reported by each reporting jurisdiction) myself, just pull the pre-calculated total from the RegionSummary section.
                                   // Since we determined there's not multiple races, I can safely assume the RegionSummary only has one race in it.
                                   racesInThisRegion = filteredRegionNodes[0].RegionSummary.Race;
                               }
                           }

                           // If there is more than one race, there can't be one distinct leader
                           if (isMultipleRaces)
                               resultScenario = resultScenarios.multipleLeaders;

                           // There is only one race for this office category in this region. We have a shot at finding a distinct leader
                           else {
                               var candidatesInfo = racesInThisRegion.Candidates.Candidate;
                               var isMultipleCandidates = candidatesInfo.length !== undefined && candidatesInfo.length > 1;
                               var leadCandidate = isMultipleCandidates ? candidatesInfo[0] : candidatesInfo;
                               var secondCandidate = isMultipleCandidates ? candidatesInfo[1] : undefined;

                               // Pull votes data
                               var leadCandidateVotes = leadCandidate.TOTAL_VOTES !== undefined ? leadCandidate.TOTAL_VOTES : leadCandidate.TOTAL;
                               var secondCandidateVotes;
                               if (secondCandidate !== undefined && secondCandidate !== null)
                                   secondCandidateVotes = secondCandidate.TOTAL_VOTES !== undefined ? secondCandidate.TOTAL_VOTES : secondCandidate.TOTAL;

                               // No votes have come in yet. No one can be leading
                               if (leadCandidateVotes === "0")
                                   resultScenario = resultScenarios.noResults;

                               // Tie
                               else if (secondCandidateVotes !== undefined && secondCandidateVotes !== null && leadCandidateVotes === secondCandidateVotes)
                                   resultScenario = resultScenarios.tie;

                               // Single, distinct leader
                               else {
                                   resultScenario = resultScenarios.distinctLeader;
                                   leaderPartyId = leadCandidate.POLITICALPARTYID;
                               }
                           }
                       }

                       return {
                           scenario: resultScenario,
                           leaderPartyId: leaderPartyId
                       };
                   }
               };
           }]);

}());